# Description: construct a call in R to Shiny
# Input:
#   type: the shinyTex function type, e.g., shinySlider (used only for error messages)
#   funName: the Shiny function name for which a call is to be constructed
#   options: a named list comprising the arguments to pass to the function
#   required: a character vector containing the names of the required arguments for the function
#   requiredTypes: a character vector of types ('c','n','l', 'cv') matching 'required'
#   optional: a character vector containing the names of the optional arguments for the function
#   optionalTypes: a character vector of types ('c','n','l') matching 'optional'
#
# Value: a string containing a complete R-style function call (or other UI code) with approriate arguments
#
# Details: called by processCommands()
#
makeCall = function(type, funName, options, required, requiredTypes, 
                    optional, optionalTypes, lineNumOffset, defaults=NULL, 
                    globalInfoEnv) {

  options = parseOptions(type, options, required, requiredTypes, 
                          optional, optionalTypes, lineNumOffset, defaults)

  # Store all "inputId" values and disallow re-use of the same "inputId" across inputs
  thisInputId = options[["inputId"]]
  if (!is.null(thisInputId)) {
    temp = objects(envir=globalInfoEnv)
    if (is.na(match("inputIds", temp))) {
      inputIds = NULL
    } else {
      inputIds = get("inputIds", envir=globalInfoEnv)
    }
    if (!is.null(thisInputId)) {
      bad = which(thisInputId == inputIds[,"inputId"])
      if (length(bad) > 0) {
        stop("@", lineNumOffset, " inputId = ", thisInputId, 
             " but this duplicates the inputId at line ", inputIds[bad, "lineNum"])
      }
    }
    inputIds = rbind(inputIds, data.frame(inputId=I(thisInputId), lineNum=lineNumOffset,
                                          type=I(type)))
    assign("inputIds", inputIds, envir=globalInfoEnv)
  }

  # Store all dropdown choices (for checking conditional panels)
  if (type == "shinyDropdown") {
    temp = ls(envir=globalInfoEnv)
    if (is.na(match("dropdownChoices", temp))) {
      dropdownChoices = NULL
    } else {
      dropdownChoices = get("dropdownChoices", envir=globalInfoEnv)
    }
    dropdownChoices = c(dropdownChoices, 
                        list(list(lineNum=lineNumOffset,
                                  choices=options$choices)))
    names(dropdownChoices)[length(dropdownChoices)] = options$inputId
    assign("dropdownChoices", dropdownChoices, envir=globalInfoEnv)
  }


  # Construct UI code for this input
  optTxt = sapply(1:length(options), 
                  function(i) {
                    name = names(options)[i]
                    opt = options[[i]]
                    if (is.character(opt)) {
                      if (length(opt)==1) {
                        return(paste0(name, '="', opt, '"'))
                      } else {
                        return(paste0(name, '=c("',
                                      paste0(opt, collapse='", "'),
                                      '")'))
                      }
                    } else if (is.numeric(opt) && length(opt)>1) {
                      return(paste0(name, '=c(',
                                    paste0(opt, collapse=', '),
                                    ')'))
                      
                    } else {
                      paste0(name, '=', opt)
                    }
                  })
  str = paste0(funName, '(', paste(optTxt, collapse=', '), '),')
  
  return(str)
} # end makeCall()
